﻿using System.Collections;
using System.Collections.Generic;
using System;
using UnityEngine;
using UnityEngine.Rendering;

namespace Mod
{
    public struct ModAssets
    {
        public static Sprite Needle;
        public static Sprite needle;
        public static Sprite Gun;
        public static Sprite gun;
        public static Sprite Grenade;
        public static Sprite grenade;
        public static Sprite Launcher;
        public static Sprite launcher;
    }

    public class Mod : MonoBehaviour
    {
        public static void Main()
        {
            ModAssets.Needle = ModAPI.LoadSprite("needle.png");
            ModAssets.needle = ModAPI.LoadSprite("needle_.png");
            ModAssets.Gun = ModAPI.LoadSprite("gun.png");
            ModAssets.gun = ModAPI.LoadSprite("gun_.png");
            ModAssets.Grenade = ModAPI.LoadSprite("Grenade.png");
            ModAssets.grenade = ModAPI.LoadSprite("Grenade_.png");
            ModAssets.Launcher = ModAPI.LoadSprite("Grenade Launcher.png");
            ModAssets.launcher = ModAPI.LoadSprite("Grenade Launcher_.png");

            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = "Syringe Gun",
                DescriptionOverride = "A gun that can fire syringe! Change the syringe that will be fired by using the gun to toch other syringe or liquid container",
                CategoryOverride = ModAPI.FindCategory("Chemistry"),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/SyringeGun.png"), 
                AfterSpawn = (Instance) =>
                {
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAssets.Gun;
                    Instance.GetComponent<PhysicalBehaviour>().HoldingPositions = new Vector3[]{new Vector3(-2f * ModAPI.PixelSize,-1f * ModAPI.PixelSize,0f)};
                    Instance.FixColliders();

                    Instance.GetOrAddComponent<SyringeGunBehaviour>();
                }
            });

            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = "Serum Grenade Launcher Gun",
                DescriptionOverride = "A gun that can fire syringe! Change the syringe that will be fired by using the gun to toch other syringe or liquid container",
                CategoryOverride = ModAPI.FindCategory("Chemistry"),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/SyringeGrenadeLauncher.png"), 
                AfterSpawn = (Instance) =>
                {
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAssets.Launcher;
                    Instance.GetComponent<PhysicalBehaviour>().HoldingPositions = new Vector3[]{new Vector3(-2f * ModAPI.PixelSize,-1f * ModAPI.PixelSize,0f)};
                    Instance.FixColliders();

                    Instance.GetOrAddComponent<SyringeBombLauncher>();
                }
            });

        ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Rod"),
                NameOverride = "Serum Attachment",
                DescriptionOverride = "Use this to give sharp object serum effects",
                CategoryOverride = ModAPI.FindCategory("Chemistry"),
                ThumbnailOverride = ModAPI.LoadSprite("sprites/Plug.png"), 
                AfterSpawn = (Instance) =>
                {
                    Instance.GetOrAddComponent<Attachment>();
                    Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Plug.png");
                    Instance.FixColliders();
                }
            });
        }
    }

    public class Attachment : MonoBehaviour
    {
        private void OnCollisionEnter2D(Collision2D other) {
            if(other.gameObject.HasComponent<AxeBehaviour>() || other.gameObject.HasComponent<SyringeBehaviour>()){
                return;
            }

            PhysicalProperties properties = other.gameObject.GetComponent<PhysicalBehaviour>().Properties;
            if(properties.Sharp || properties.SharpAxes.Length > 0){
                other.gameObject.AddComponent<AxeBehaviour>();
                Destroy(this.gameObject);
            }
        }
    }

    public class SyringeGunBehaviour : LiquidTank
    {
        public SpriteRenderer spriterender;

        private void Awake() {
            UseEventTrigger useTrigger = base.gameObject.AddComponent<UseEventTrigger>();
            useTrigger.Action = () =>
            {
                Fire();
            };

            SetMask();

            liquids.Add(Liquid.GetLiquid("KNOCKOUT POISON"),1f);
        }

        public virtual void Fire(){
            GameObject Syringe = InstantiateNeedle();
            Syringe.transform.position = base.transform.position + Vector3.Scale(new Vector3(ModAPI.PixelSize * 0.5f,ModAPI.PixelSize * 1.5f,0f) , base.transform.localScale);
            Syringe.transform.rotation = base.transform.rotation;
            if(base.transform.localScale.x > 0){
                Syringe.transform.Rotate(Vector3.back, 90f);
            }
            else{
                Syringe.transform.Rotate(Vector3.back, -90f);
            }
            foreach(Collider2D coll in base.GetComponentsInChildren<Collider2D>()){
                foreach(Collider2D Coll in Syringe.GetComponentsInChildren<Collider2D>()){
                    Physics2D.IgnoreCollision(coll,Coll,ignore:true);
                }
            }
            foreach(Rigidbody2D rig in Syringe.GetComponentsInChildren<Rigidbody2D>()){
                rig.collisionDetectionMode = CollisionDetectionMode2D.Continuous;
                rig.mass = rig.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass;
                rig.mass *= 0.2f;
                rig.AddForce(4500f * (base.transform.rotation * Vector2.right) * rig.mass * base.transform.localScale.x * base.transform.localScale.x * base.transform.localScale.x);
            }
        }

        public virtual void SetMask(){
            GameObject Gun = new GameObject("Gun");
            Gun.transform.position = base.transform.position;
            Gun.transform.rotation = base.transform.rotation;
            Gun.transform.localScale = base.transform.localScale;
            Gun.transform.SetParent(base.transform);
            
            spriterender = Gun.AddComponent<SpriteRenderer>();
            spriterender.sprite = ModAssets.gun;
            spriterender.sortingOrder += 1;
        }

        private void Update() {
            Vector4 LiquidColor = Vector4.zero;
            foreach(Liquid liquid in liquids.Keys){
                LiquidColor += (Vector4)liquid.Color * liquids[liquid];
            }
            spriterender.color = LiquidColor;
        }

        private void OnCollisionEnter2D(Collision2D other) {
            if(!other.gameObject.HasComponent<LimbBehaviour>() && other.gameObject.TryGetComponent<BloodContainer>(out BloodContainer container)){
                SyncLiquids(container);
            }
        }

        public virtual GameObject InstantiateNeedle(){
            ModAPI.FindSpawnable("Knife").Prefab.GetComponent<PhysicalBehaviour>().SpawnSpawnParticles = false;
            GameObject needle = Instantiate(ModAPI.FindSpawnable("Knife").Prefab);
            ModAPI.FindSpawnable("Knife").Prefab.GetComponent<PhysicalBehaviour>().SpawnSpawnParticles = true;
            needle.GetComponent<SpriteRenderer>().sprite = ModAssets.Needle;
            needle.transform.localScale = new Vector3(Mathf.Abs(base.transform.localScale.y),Mathf.Abs(base.transform.localScale.x),1f);
            needle.AddComponent<NeedleBehaviour>().liquids = new Dictionary<Liquid,float>(liquids);
            needle.GetComponent<NeedleBehaviour>().Ready = true;
            needle.FixColliders();
            return needle;
        }
    }

    public class SyringeBombLauncher : SyringeGunBehaviour
    {   
        public override GameObject InstantiateNeedle(){
            ModAPI.FindSpawnable("Rod").Prefab.GetComponent<PhysicalBehaviour>().SpawnSpawnParticles = false;
            GameObject needle = Instantiate(ModAPI.FindSpawnable("Rod").Prefab);
            ModAPI.FindSpawnable("Rod").Prefab.GetComponent<PhysicalBehaviour>().SpawnSpawnParticles = true;
            needle.GetComponent<SpriteRenderer>().sprite = ModAssets.Grenade;
            needle.GetComponent<SpriteRenderer>().sortingOrder -= 1;
            needle.transform.localScale = new Vector3(Mathf.Abs(base.transform.localScale.y),Mathf.Abs(base.transform.localScale.x),1f);
            needle.AddComponent<SyringeBomb>().liquids = new Dictionary<Liquid,float>(liquids);
            needle.FixColliders();
            return needle;
        }

        public override void SetMask(){
            if(spriterender){
                return;
            }
            GameObject Gun = new GameObject("Gun");
            Gun.transform.position = base.transform.position;
            Gun.transform.rotation = base.transform.rotation;
            Gun.transform.localScale = base.transform.localScale;
            Gun.transform.SetParent(base.transform);
            
            spriterender = Gun.AddComponent<SpriteRenderer>();
            spriterender.sprite = ModAssets.launcher;
            spriterender.sortingOrder += 1;
        }

        public override void Fire(){
            GameObject Syringe = InstantiateNeedle();
            Syringe.transform.position = base.transform.position + Vector3.Scale(new Vector3(ModAPI.PixelSize * 8.5f,ModAPI.PixelSize * 3.5f,0f) , base.transform.localScale);
            Syringe.transform.rotation = base.transform.rotation;
            if(base.transform.localScale.x > 0){
                Syringe.transform.Rotate(Vector3.back, 90f);
            }
            else{
                Syringe.transform.Rotate(Vector3.back, -90f);
            }
            foreach(Collider2D coll in base.GetComponentsInChildren<Collider2D>()){
                foreach(Collider2D Coll in Syringe.GetComponentsInChildren<Collider2D>()){
                    Physics2D.IgnoreCollision(coll,Coll,ignore:true);
                }
            }
            foreach(Rigidbody2D rig in Syringe.GetComponentsInChildren<Rigidbody2D>()){
                rig.collisionDetectionMode = CollisionDetectionMode2D.Continuous;
                rig.mass = rig.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass;
                rig.mass *= 0.2f;
                rig.AddForce(4500f * (base.transform.rotation * Vector2.right) * rig.mass * base.transform.localScale.x * base.transform.localScale.x * base.transform.localScale.x);
            }
        }
    }

    public class SyringeBomb : LiquidTank
    {
        PhysicalBehaviour Phys;
        GameObject Needle;

        private void Awake() {
            Phys = GetComponent<PhysicalBehaviour>();
            Phys.StabCausesWound = false;

            Needle = new GameObject("Bomb_");
            Needle.transform.SetParent(base.transform);
            Needle.transform.rotation = base.transform.rotation;
            Needle.transform.position = base.transform.position;
            Needle.transform.localScale = Vector3.one;
            SpriteRenderer spriterender = Needle.AddComponent<SpriteRenderer>();
            spriterender.sprite = ModAssets.grenade;
        }

        private void LateUpdate() {
            Needle.GetComponent<SpriteRenderer>().color = GetComputedColor();
        }

        private void OnCollisionEnter2D(Collision2D other) {
            BreakSyringe();
        }

        protected virtual void BreakSyringe()
        {
            SyringeExplosionBehaviour component = Instantiate(Resources.Load<GameObject>("Prefabs/SyringeExplosion"), base.transform.position, Quaternion.identity).GetComponent<SyringeExplosionBehaviour>();
            component.Colour = GetComputedColor();
            component.Liquids = liquids.Keys;
            component.Amount = 1f * Mathf.Abs(base.transform.localScale.x);
            component.Radius *= Mathf.Abs(base.transform.localScale.x);
            Destroy(base.gameObject);
        }
    }

    public class AxeBehaviour : LiquidTank
    {
        PhysicalBehaviour Phys;
        public GameObject Mask;
        public Sprite mask;
        public bool Ready = false;

        private void Awake() {
            Phys = GetComponent<PhysicalBehaviour>();
            liquids.Add(Liquid.GetLiquid("KNOCKOUT POISON"),1f);

            Mask = new GameObject(base.gameObject.name + "_");
            Mask.AddComponent<SpriteRenderer>().sprite = base.GetComponent<SpriteRenderer>().sprite;
            base.GetComponent<SpriteRenderer>().sortingOrder -= 1;
            Mask.transform.position = base.transform.position;
            Mask.transform.rotation = base.transform.rotation;
            Mask.transform.SetParent(base.transform);
            Mask.transform.localScale = Vector3.one;
            this.GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(
                new ContextMenuButton(
                    "Remove the serum effect",
                    "Remove the serum effect",
                    "Remove the serum effect",
                    () =>
                    {
                        Destroy(Mask);
                        Destroy(this);
                        ModAPI.Notify("Remove the serum effect!");
                    }
                )
            );
        }

        private void Update() {
            if(Phys.penetrations.Count == 0){
                return;
            }

            foreach(PhysicalBehaviour.Penetration penetrate in Phys.penetrations){
                if(penetrate.Victim.TryGetComponent<LimbBehaviour>(out LimbBehaviour limb)){
                    foreach(Liquid liquid in liquids.Keys){
                        limb.CirculationBehaviour.AddLiquid(liquid, Time.deltaTime / Phys.penetrations.Count * 10f * liquids[liquid]);
                    }
                }
            }
        }

        private void OnCollisionEnter2D(Collision2D other) {
            if(!other.gameObject.HasComponent<LimbBehaviour>() && other.gameObject.TryGetComponent<BloodContainer>(out BloodContainer container)){
                SyncLiquids(container);
            }
        }



        private void LateUpdate() {
            Mask.GetComponent<SpriteRenderer>().color = GetComputedColor();
        }
    }

    public class NeedleBehaviour : LiquidTank
    {
        PhysicalBehaviour Phys;
        public float LiquidAmount = 1f;
        public GameObject Needle;
        public GameObject NeedleMask;
        public List<PersonBehaviour> DisableCollide = new List<PersonBehaviour>();
        private bool stabstate;
        public Sprite mask;
        public bool Ready = false;

        private void Awake() {
            base.gameObject.AddComponent<DebrisComponent>();

            Phys = GetComponent<PhysicalBehaviour>();
            Phys.StabCausesWound = false;

            Needle = new GameObject("needle_");
            Needle.transform.SetParent(base.transform);
            Needle.transform.rotation = base.transform.rotation;
            Needle.transform.position = base.transform.position;
            Needle.transform.localScale = Vector3.one;
            SpriteRenderer spriterender = Needle.AddComponent<SpriteRenderer>();
            spriterender.sprite = ModAssets.needle;
            spriterender.maskInteraction = SpriteMaskInteraction.VisibleInsideMask;

            NeedleMask = new GameObject("needle_mask");
            NeedleMask.transform.SetParent(base.transform);
            NeedleMask.transform.rotation = base.transform.rotation;
            NeedleMask.transform.position = base.transform.position;
            NeedleMask.transform.localScale = Vector3.one;
            SpriteMask mask = NeedleMask.AddComponent<SpriteMask>();
            mask.sprite = ModAssets.needle;
            mask.isCustomRangeActive = true;
            mask.backSortingLayerID = SortingLayer.NameToID("Default");
            mask.frontSortingLayerID = SortingLayer.NameToID("Default");
            mask.backSortingOrder = -5;
            mask.frontSortingOrder = 5;
        }

        private void Update() {
            if(Phys.penetrations.Count == 0 || LiquidAmount < 0){
                return;
            }

            LiquidAmount -= Time.deltaTime * 0.75f;
            foreach(PhysicalBehaviour.Penetration penetrate in Phys.penetrations){
                if(penetrate.Victim.TryGetComponent<LimbBehaviour>(out LimbBehaviour limb)){
                    foreach(Liquid liquid in liquids.Keys){
                        limb.CirculationBehaviour.AddLiquid(liquid, Time.deltaTime / Phys.penetrations.Count * 1.5f * liquids[liquid]);
                    }
                }
            }
        }

        private void LateUpdate() {
            Needle.GetComponent<SpriteRenderer>().color = GetComputedColor();
            NeedleMask.transform.localPosition = Vector3.up * (1f - LiquidAmount) * ModAPI.PixelSize * 8;
        }
    }

    public class LiquidTank : MonoBehaviour
    {
        public Dictionary<Liquid,float> liquids = new Dictionary<Liquid,float>();

        public ConservedDistribution[] ConservedLiquids;

        public struct ConservedDistribution
        {
            public string ID;
            public float Distribution;
        }

        public Color GetComputedColor()
        {
            Vector4 LiquidColor = Vector4.zero;
            foreach(Liquid liquid in liquids.Keys){
                LiquidColor += (Vector4)liquid.Color * liquids[liquid];
            }
            return (Color)LiquidColor;
        }
                
        public void SyncLiquids(BloodContainer container){
            liquids.Clear();
            foreach(Liquid liquid in container.LiquidDistribution.Keys){
                liquids.Add(liquid,container.LiquidDistribution[liquid].Raw/container.TotalLiquidAmount);
            }
        }

        public virtual void OnBeforeSerialise()
        {
            int i = 0;
            List<ConservedDistribution> distributions = new List<ConservedDistribution>();
            foreach(Liquid liquid in liquids.Keys){
                distributions.Add(new ConservedDistribution{ID = Liquid.GetIdentity(liquid),Distribution = liquids[liquid]});
            }
            ConservedLiquids = distributions.ToArray();
        }

        public virtual void OnAfterDeserialise(List<GameObject> gameObjects)
	    {
            liquids.Clear();
            foreach(ConservedDistribution distribution in ConservedLiquids){
                liquids.Add(Liquid.GetLiquid(distribution.ID),distribution.Distribution);
            }
        }
    }
}
// Originally uploaded by 'NeitherFishNorCat'. Do not reupload without their explicit permission.
